/** 
	A "hello" program in Qt
	that use OpenGL drawing primitives
	File: quad.cpp

	Fraga 18/09/2014
**/
#include <qpainter.h>
#include <ctype.h>
#include <glu.h>
#include "pantalla.h"

Pantalla::Pantalla( QWidget *parent )
		: QGLWidget( parent )
//  : QGLWidget( QGLFormat( QGL::SingleBuffer|QGL::NoDepthBuffer ), parent )
{
	signo = 1;	

	// setMouseTracking( TRUE );

	// setFocusPolicy( QWidget::StrongFocus);
	setFocusPolicy( Qt::StrongFocus);

	timer = new QTimer( this );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	xrot = yrot = 0;
	angle = 0;

	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );

	crea_cono( cono1, norm1, 10, 0.5, 0.6 );
}


void Pantalla::timerDone( )
{
	angle += 10.0*signo;

	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;
	
	updateGL( );
}

void Pantalla::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
	xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

void Pantalla::keyPressEvent ( QKeyEvent * e )
{
	switch ( e->key() ) {
		case '+':   
			signo = 1;
			break;
		case '-':  
			signo = -1;
			break;
	}
}

//
// This method puts the initial coordinates each time window is resized
//
void Pantalla::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	float a = 1.0;

	if( w > h )
		glOrtho (-a*aspect, a*aspect, -a, a, -a, a );
	else
		glOrtho ( -a, a, -a/aspect, a/aspect, -a, a );
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */

	emit changeSize ( );
}

QSizePolicy Pantalla::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

void Pantalla::paintGL(void)
{
	/** Clear all pixels **/
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */

	glRotatef ( xrot , 1.0, 0.0, 0.0);
	glRotatef ( yrot , 0.0, 1.0, 0.0);

	dibuja_cono( cono1, norm1, 10 );

	// swapBuffers( );
}

void Pantalla::initializeGL( ) 
{
	glClearColor ( 0.8, 0.8, 0.8, 0.0);	// Background to a grey tone

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0 );

	glEnable (GL_DEPTH_TEST);

	// Por defecto es GL_SMOOTH
	// glShadeModel( GL_SMOOTH );
	// glShadeModel( GL_FLAT );

	// Para habilitar la ilumincacin
	// necesitamos definir materiales y fuentes de luz...
	// Y algunos parmetros ms 

	// Las fuentes de luz deben ser blancas
	float luzblanca[4] = { 1.0, 1.0, 1.0, 1.0 };
	float pos1[4] = { 2.0, 2.0, 2.0, 1.0 };

	glLightfv( GL_LIGHT0, GL_POSITION, pos1 );
	glLightfv( GL_LIGHT0, GL_DIFFUSE, luzblanca );
	glLightfv( GL_LIGHT0, GL_SPECULAR, luzblanca );

	// Definimos el material
	GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };  // k_esp
	GLfloat mat_shininess[] = { 80.0 };    // el exponent n_s del cos phi
	GLfloat k_ambiente[] = { 0.0, 0.0, 1.0, 1.0 };  // k_d 

	glMaterialfv( GL_FRONT, GL_SPECULAR, mat_specular );
	glMaterialfv( GL_FRONT, GL_SHININESS, mat_shininess );
	glMaterialfv( GL_FRONT, GL_AMBIENT, k_ambiente );


	// Configuramos
	GLfloat lmodel_ambient[] = { 0.1, 0.1, 0.1, 1.0 };  // k_a
	glLightModelfv( GL_LIGHT_MODEL_AMBIENT, lmodel_ambient );
	glEnable (GL_LIGHTING);
    glEnable (GL_LIGHT0);
}


void Pantalla::crea_cono( double c[][3], double norm[][3],  int n, double r, double a )
{
	int i;
	double theta, alfa;

	theta = 2.0*3.141592654/n;
	alfa = atan( r/a );

	for( i=0; i<n; i++ ) {
		c[i][0] = r*cos( i*theta );
		c[i][1] = 0.0;
		c[i][2] = r*sin( i*theta );

		norm[i][0] = cos( i*theta ) * cos( alfa );
		norm[i][1] = sin( alfa );
		norm[i][2] = sin( i*theta ) * cos( alfa );
	}

	// El vrtice de la altura:
	c[n][0] = 0.0;
	c[n][1] = a;
	c[n][2] = 0.0;

	norm[n][0] = 0.0;
	norm[n][1] = 1.0;
	norm[n][2] = 0.0;
}


void Pantalla::dibuja_cono( double c[][3], double norm[][3], int n )
{
	float color[3] = { 1.0, 0.0, 0.0 };   // rojo 
	int i;


	glBegin( GL_TRIANGLE_FAN );

	// glColor3f( 1.0, 1.0, 1.0 );
	glNormal3dv( &norm[ n ][0] );
	glVertex3dv( &c[ n ][0] );

	for( i=0; i<n; i++ ) {
		// glColor3fv( &color[0] );
		glNormal3dv( &norm[ i ][0] );
		glVertex3dv( &c[ i ][0] );
	}
	glNormal3dv( &norm[ 0 ][0] );
	glVertex3dv( &c[ 0 ][0] );

	glEnd( );
}

